#include "SnoopAdapter.h"

REGISTER_METACLASS(SnoopAdapter, SnoopCapture)

// ----------------------------------------------------------------------------
// SnoopAdapter
// ----------------------------------------------------------------------------
SnoopAdapter::SnoopAdapter(void* owner) : SnoopPcap(owner)
{
  SnoopAdaptersInfo::classInitialize();
  m_adapterIndex = SNOOP_INVALID_ADAPTER_INDEX;
}

SnoopAdapter::~SnoopAdapter()
{
  close();
  SnoopAdaptersInfo::classFinalize();
}

bool SnoopAdapter::doOpen()
{
  SnoopAdapterInfo* adapterInfo;

  if (m_adapterIndex == SNOOP_INVALID_ADAPTER_INDEX)
  {
    SET_ERROR(VDError, "invalid adapter index(-1)", VDERR_INVALID_INDEX);
    return false;
  }
  adapterInfo = snoopGetAdapterInfo(m_adapterIndex);
  if (adapterInfo == NULL)
  {
    SET_ERROR(VDError, format("can not find adapterInfo(adapter index is %d", m_adapterIndex), VDERR_OBJECT_IS_NULL);
    return false;
  }
  if (adapterInfo->dev == NULL)
  {
    SET_ERROR(VDError, "dev is NULL", VDERR_OBJECT_IS_NULL);
    return false;
  }
  if (!pcapOpen((char*)adapterInfo->name.c_str(), NULL, adapterInfo->dev))
  {
    return false;
  }

  return SnoopPcap::doOpen();
}

bool SnoopAdapter::doClose()
{
  return SnoopPcap::doClose();
}

void SnoopAdapter::setAdapterIndex(int value)
{
  if (m_adapterIndex == value) return;
  SnoopAdapterInfo* adapterInfo = snoopGetAdapterInfo(value);
  if (adapterInfo != NULL) m_adapterIndex = adapterInfo->index;
}

void SnoopAdapter::load(VDXML* xml)
{
  SnoopPcap::load(xml);
  adapterIndex = xml->getInt("adapterIndex", adapterIndex);
}

void SnoopAdapter::save(VDXML* xml)
{
  SnoopPcap::save(xml);
  xml->setInt("adapterIndex", adapterIndex);
}

// ----------------------------------------------------------------------------
// This directive is used to determine that GetBestInterface API is used to detect network interface
#define USE_GETBESTINTERFACE
// ----------------------------------------------------------------------------

#ifndef USE_GETBESTINTERFACE

#include "VDTCPClient.h"

// ----------------------------------------------------------------------------
// GeneratePacketThread
// ----------------------------------------------------------------------------
class GeneratePacketThread : public VDThread
{
public:
  GeneratePacketThread()          {}
  virtual ~GeneratePacketThread() { close(); }
  bool close()
  {
    if (tcpClient.tcpSession != NULL)
    {
      SOCKET socketHandle = tcpClient.tcpSession->socketHandle;
      closesocket(socketHandle);
      shutdown(socketHandle, SD_BOTH);
    }
    tcpClient.close();
    return VDThread::close();
  }
public:
  string host;
protected:
  VDTCPClient tcpClient;
  virtual void run()
  {
    tcpClient.host = this->host;
    tcpClient.port = 80;
    tcpClient.open();
  }
};

// ----------------------------------------------------------------------------
// SnoopAutoDetectAdapter
// ----------------------------------------------------------------------------
class SnoopAutoDetectAdapter
{
protected:
  int         m_adapterIndex;
  VDTCPClient m_tcpClient;
  VDEvent*    m_event;
public:
  SnoopAutoDetectAdapter();
  virtual ~SnoopAutoDetectAdapter();
  int detect(string host);
protected:
  static void _adapter_OnCapture(
    SnoopCapture* capture,
    SnoopPacket*   packet);
};

SnoopAutoDetectAdapter::SnoopAutoDetectAdapter()
{
  m_adapterIndex = SNOOP_INVALID_ADAPTER_INDEX;
  m_event        = new VDEvent(NULL, false, false, NULL);
}

SnoopAutoDetectAdapter::~SnoopAutoDetectAdapter()
{
  SAFE_DELETE(m_event);
}

int SnoopAutoDetectAdapter::detect(string host)
{
  if (m_adapterIndex != SNOOP_INVALID_ADAPTER_INDEX)
  {
    return m_adapterIndex;
  }

  //
  // Create and open all adapters
  //
  int count = snoopGetAdapterCount();
  VDPList<SnoopAdapter*> adapters;
  for (int i = 0; i < count; i++)
  {
    SnoopAdapter* adapter = new SnoopAdapter;
    adapter->owner = this;
    adapters.add(adapter);

    adapter->onCapture  = _adapter_OnCapture;
#ifdef SNOOP_THREAD_SAFE_SUPPORT
    adapter->threadSafe = false;
#endif // SNOOP_THREAD_SAFE_SUPPORT
    adapter->filter = format("(host %s) and (arp or udp or tcp)", host.c_str()); // arp(finding gateway), udp(resolve domainname), tcp(connect)
    adapter->adapterIndex = i;
    if (!adapter->open())
    {
      LOG_ERROR("[SnoopAdapter.cpp] SnoopAutoDetectAdapter::detect can not open adapter(%d)", i);
    }
  }

  //
  // Generate packet
  //
  GeneratePacketThread* thread = new GeneratePacketThread;
  thread->host = host;
  thread->open();

  //
  // Wait
  //
  m_event->waitFor(5000);

  adapters.clear();
  thread->close();
  delete thread;
  return m_adapterIndex;
}

void SnoopAutoDetectAdapter::_adapter_OnCapture(
  SnoopCapture* capture,
  SnoopPacket*   packet)
{
  SnoopAdapter* adapter = (SnoopAdapter*)capture;
  SnoopAutoDetectAdapter* detect = (SnoopAutoDetectAdapter*)adapter->owner;
  if (detect->m_adapterIndex != SNOOP_INVALID_ADAPTER_INDEX) return;
  detect->m_adapterIndex = adapter->adapterIndex;
  LOG_INFO("[SnoopAdapter.cpp] SnoopAutoDetectAdapter::_adapter_OnCapture adapterIndex=%d(%s)",
    detect->m_adapterIndex, snoopGetAdapterInfo(detect->m_adapterIndex)->description);
  detect->m_event->setEvent();
}

#else  USE_GETBESTINTERFACE

// ----------------------------------------------------------------------------
// SnoopAutoDetectAdapter
// ----------------------------------------------------------------------------
class SnoopAutoDetectAdapter
{
public:
  int detect(string host);
};

int SnoopAutoDetectAdapter::detect(string host)
{
  IPAddr ipAddr;
  DWORD  bestIfIndex;
  ipAddr = htonl(str_to_ip(VDNet::getIP(host)));
  DWORD res = GetBestInterface(ipAddr, &bestIfIndex);
  if (res != NO_ERROR)
  {
    LOG_FATAL("[SnoopAdapter.cpp] SnoopAutoDetectAdapter::detect GetBestInterface return %d", res);
    return SNOOP_INVALID_ADAPTER_INDEX;
  }

  for (int i = 0; i < snoopGetAdapterCount(); i++)
  {
    SnoopAdapterInfo* snoopAdapterInfo = snoopGetAdapterInfo(i);
	if (snoopAdapterInfo->ipAdapterInfo == NULL) continue;
    if (snoopAdapterInfo->ipAdapterInfo->Index == bestIfIndex)
      return i;
  }

  LOG_FATAL("[SnoopAdapter.cpp] SnoopAutoDetectAdapter::detect can not find appropriate adapter for %s", host.c_str());
  return SNOOP_INVALID_ADAPTER_INDEX;;
}

#endif USE_GETBESTINTERFACE

int SnoopAdapter::getBestAdapterIndex(string host)
{
  if (host == "") host = "8.8.8.8"; // Well-knowon Google DNS Server IP

  SnoopAutoDetectAdapter detect;
  return detect.detect(host);
};