#include "SnoopAdapterInfo.h"

// ----------------------------------------------------------------------------
// SnoopAdapterInfo
// ----------------------------------------------------------------------------
SnoopAdapterInfo::SnoopAdapterInfo()
{
  index         = SNOOP_INVALID_ADAPTER_INDEX;
  name          = "";
  description   = "";
  dev           = NULL;
  ipAdapterInfo = NULL;
}

SnoopAdapterInfo::~SnoopAdapterInfo()
{
}

bool SnoopAdapterInfo::operator == (SnoopAdapterInfo& rhs)
{
  if (this->index != rhs.index) return false;
  if (name != rhs.name) return false;
  if (description != rhs.description) return false;
  // if (this->dev != rhs.dev) return false;
  return true;
}

bool SnoopAdapterInfo::operator != (SnoopAdapterInfo& rhs)
{
  return !(*this == rhs);
}

void SnoopAdapterInfo::load(VDXML* xml)
{
  index = xml->getInt("index", index);
  name = xml->getStr("name", name);
  description = xml->getStr("description", description);
}

void SnoopAdapterInfo::save(VDXML* xml)
{
  xml->setInt("index", index);
  xml->setStr("name", name);
  xml->setStr("description", description);
}

// ----------------------------------------------------------------------------
// SnoopAdaptersInfo
// ----------------------------------------------------------------------------
SnoopAdaptersInfo::SnoopAdaptersInfo()
{
  items.clear();
  allDevs           = NULL;
  allIPAdaptersInfo = NULL;
}

SnoopAdaptersInfo::~SnoopAdaptersInfo()
{
  finalize();
}

void SnoopAdaptersInfo::initialize()
{
  //
  // Initialize allDevs using pcap API.
  //
  if (allDevs != NULL) return;

  char errBuf[PCAP_ERRBUF_SIZE];

  int i = pcap_findalldevs_ex("rpcap://", NULL, &allDevs, errBuf);
  if (i != 0) // if error occured
  {
    LOG_ERROR("[SnoopAdapterInfo.cpp] SnoopAdaptersInfo::initialize error in pcap_findalldevs_ex (%s)", errBuf);
    return;
  }

  pcap_if_t* dev = allDevs;
  i = 0;
  while (dev != NULL)
  {
    SnoopAdapterInfo* adapterInfo = new SnoopAdapterInfo;

    adapterInfo->index = i;
    adapterInfo->name = dev->name;
    adapterInfo->description = dev->description;
    adapterInfo->dev = dev;

    items.add(adapterInfo);

    dev = dev->next;
    i++;
  }

  //
  // Initialize allAdapterInfos using IPHelper API.
  //
  ULONG outBufLen = 0;
  DWORD res = GetAdaptersInfo(allIPAdaptersInfo, &outBufLen);
  if (res == ERROR_BUFFER_OVERFLOW)
  {
    allIPAdaptersInfo = (PIP_ADAPTER_INFO)malloc(outBufLen);
    res = GetAdaptersInfo(allIPAdaptersInfo, &outBufLen);
  }
  if (res != ERROR_SUCCESS)
  {
    LOG_ERROR("[SnoopAdapterInfo.cpp] SnoopAdaptersInfo::initialize GetAdaptersInfo return %d(0x%x)", res, res);
    return;
  }

  for (int i = 0; i < items.count(); i++)
  {
    string adapterName = items.at(i)->name;

    PIP_ADAPTER_INFO p = allIPAdaptersInfo;
    while (p != NULL)
    {
      if (strstr(adapterName.c_str(), p->AdapterName) != NULL)
        break;
      p = p->Next;
    }
    if (p == NULL)
    {
      LOG_DEBUG("[SnoopAdapterInfo.cpp] SnoopAdaptersInfo::initialize can not find adapter(%d) %s", i, adapterName.c_str());
    }
    items.at(i)->ipAdapterInfo = p;
  }
}

void SnoopAdaptersInfo::finalize()
{
  items.clear();

  //
  // Finalize allDevs
  //
  if (allDevs != NULL)
  {
    pcap_freealldevs(allDevs);
    allDevs = NULL;
  }

  //
  // Finalize allIPAdaptersInfo
  //
  if (allIPAdaptersInfo != NULL)
  {
    free(allIPAdaptersInfo);
    allIPAdaptersInfo = NULL;
  }
}

bool SnoopAdaptersInfo::operator == (SnoopAdaptersInfo& rhs)
{
  if (items.count() != rhs.items.count()) return false;
  for (int i = 0; i < items.count(); i++)
  {
    if (items.at(i) != rhs.items.at(i)) return false;
  }
  return true;
}

bool SnoopAdaptersInfo::operator != (SnoopAdaptersInfo& rhs)
{
  return !(*this == rhs);
}

void SnoopAdaptersInfo::load(VDXML* xml)
{
  items.clear();
  int count = xml->getInt("count", 0);
  for (int i = 0; i < count; i++)
  {
    VDXML* childXML = xml->getChild(i);

    SnoopAdapterInfo* adapterInfo = new SnoopAdapterInfo;
    adapterInfo->load(childXML);
    items.add(adapterInfo);
  }
}

void SnoopAdaptersInfo::save(VDXML* xml)
{
  xml->clearChild();
  int count = items.count();
  xml->setInt("count", count);
  for (int i = 0; i < count; i++)
  {
    VDXML* childXML = xml->addChild("SnoopAdapter");

    items.at(i)->save(childXML);
  }
}

SnoopAdaptersInfo* SnoopAdaptersInfo::g_adaptersInfo = NULL;

void SnoopAdaptersInfo::doClassInitialize()
{
  VDLog::classInitialize();

  g_adaptersInfo = new SnoopAdaptersInfo;
  g_adaptersInfo->initialize();
}

void SnoopAdaptersInfo::doClassFinalize()
{
  SAFE_DELETE(g_adaptersInfo);

  VDLog::classFinalize();
}

// ----------------------------------------------------------------------------
// Functions
// ----------------------------------------------------------------------------
int snoopGetAdapterCount()
{
  SnoopAdaptersInfo* adaptersInfo = SnoopAdaptersInfo::g_adaptersInfo;
  LOG_ASSERT(adaptersInfo != NULL); 
  if (adaptersInfo == NULL) return 0;
  return adaptersInfo->items.count();
}

SnoopAdapterInfo* snoopGetAdapterInfo(int index)
{
  SnoopAdaptersInfo* adaptersInfo = SnoopAdaptersInfo::g_adaptersInfo;
  LOG_ASSERT(adaptersInfo != NULL);
  if (adaptersInfo == NULL) return NULL;
  if (index < 0 || index >= adaptersInfo->items.count())
  {
    LOG_ERROR("[SnoopAdapterInfo.cpp] snoopGetAdapterInfo invalid adapter index(%d)", index);
    return NULL;
  }
  return adaptersInfo->items.at(index);
}

// ----------------------------------------------------------------------------
// VDAutoClassInitializer<SnoopAdaptersInfo>
// ----------------------------------------------------------------------------
#ifdef VD_USE_AUTO_CLASS_INITIALIZER
class VDAutoClassInitializer<SnoopAdaptersInfo> adaptersInfo_AutoClassInitializer;
#endif VD_USE_AUTO_CLASS_INITIALIZER